package Hanoi::Disc;

=pod

=head1 NAME

Hanoi::Disc - A disc in the tower of Hanoi problem.

=head1 SYNOPSIS

 use Hanoi::Disc;

 my Hanoi::Disc $dsc;
 $dsk  = new Hanoi::Disc( Size => $x );
 $dsk2 = new Hanoi::Disc( Size => $y );

 $dsk->set_size($x);
 $dsk2->get_size();
 $dsk->bigger_than($dsk2);
 $dsk2->smaller_than($dsk1);

=head1 DESCRIPTION

This is an OO implementation of a disc in the Tower of Hanoi problem.

This is I<overkill>... an example of when OO code is over the top.

=cut


# Class definition
use strict;
use vars qw/ $VERSION @EXPORT_OK /;

# Dependencies
use Exporter;

# Inheritence
use base qw/ Exporter /;


$VERSION   = (split(/ /, ' $Version: 1.12 $ '))[2];
@EXPORT_OK = qw/ &is_disc /;


#########################################################################
=pod

=head1 CONSTRUCTOR

=over 4

=item B<new( [ Size => $x ] )>

Creates a new Disc with size I<x> (defaults to 1).  Returns the new object if
there were no problems creating it, or undef on failure.

=cut

sub new {
    my $class = shift;
    $class    = ref($class) || $class || return undef;
    my %args  = @_;
    my $size;

    # notice we use a scalar as the object here:
    my $self  = bless \$size, $class;

    return(undef) unless ($self->set_size($args{Size} || 1));

    return $self;
}


#########################################################################
=pod

=back

=head1 METHODS

=over 4

=item B<get_size()>

Get disc size.

=cut

  sub get_size {
      my $self = shift;
      return $$self;
  }


#########################################################################
=pod

=item B<set_size()>

Set disc size.  Returns new size on sucess, undef on invalid size;

=cut

  sub set_size {
      my $self    = shift;
      my $newsize = shift;

      # size must be defined and > 0!
      unless ($newsize > 0) {
	  warn "disc size($newsize) must be greater than 0!\n";
	  return undef;
      }

      $$self = $newsize;

      return $self->get_size();
  }


#########################################################################
=pod

=item B<bigger_than( $disc2 )>

Returns true if this disc is bigger than that disc.

=cut

  sub bigger_than {
      my $d1 = shift || return;
      my $d2 = shift || return;

      return ($d1->get_size > $d2->get_size);
  }


#########################################################################
=pod

=item B<smaller_than( $disc2 )>

Returns true if this disc is smaller than that disc.

=cut

  sub smaller_than {
      my $d1 = shift || return;
      my $d2 = shift || return;

      return ($d1->get_size < $d2->get_size);
  }



#########################################################################
=pod

=back

=head1 SUBROUTINES

=over 4

=item B<is_peg( $variable )>

Returns true value if $variable is a member of the I<Hanoi::Disc> class
(or sub-class), and a false falue otherwise.

Exported with @EXPORT_OK.

=cut

  sub is_disc {
      my $variable = shift || return; # should you print a warning here?
      return UNIVERSAL::isa($variable, 'Hanoi::Disc');
  }



#########################################################################
# Embedded test script...
package main;
unless (caller) {
    my Hanoi::Disc ($d1, $d2);

    print "Disc sizes:\n";
    $d1 = new Hanoi::Disc( Size => int(rand()*10) ) || die "couldn't create disc 1!";
    print "\tD1 = ", $d1->get_size(), "\n";
    $d2 = new Hanoi::Disc() || die "couldn't create disc 2!";
    $d2->set_size(int(rand()*10)) || die "couldn't set size of disc 2!";
    print "\tD2 = ", $d2->get_size(), "\n";

    if ($d1->bigger_than($d2)) {
	print "d1 > d2!\n";
    } elsif ($d1->smaller_than($d2)) {
	print "d1 < d2!\n";
    } else {
	print "d1 == d2!\n";
    }

}



1;

__END__

#########################################################################
=pod

=back

=head1 INHERITS FROM

 

=head1 DEPENDANCIES

 none

=head1 NOTES

None.

=head1 TODO

Overload '>', '<' and '==' ops to use get_size() ?
    -spurkis

=head1 AUTHORS

 Steve Purkis <spurkis@redhotchilli.com>

=head1 SEE ALSO

 Hanoi::Peg
 Hanoi::Tower

=cut


