package Hanoi::Peg;

=pod

=head1 NAME

Hanoi::Peg - A peg in the tower of Hanoi problem.

=head1 SYNOPSIS

 use Hanoi::Peg qw/ &is_peg /;

 my $peg = new Hanoi::Peg();
 $foo->do_stuff();
 $boolean = is_foo($foo);

=head1 DESCRIPTION

Implements a single Peg from the Tower of Hanoi game.  Also includes
rules on how to move discs on and off this peg.

=cut


# Class definition
use strict;
use vars   qw/ $VERSION %FIELDS @EXPORT_OK /;
use fields qw/ discs /;

# Class dependencies
use Hanoi::Disc qw/ is_disc /;
use Exporter;

# Inheritence
use base qw/ Exporter /;


$VERSION   = (split(/ /, ' $Version: 1.12 $ '))[2];
@EXPORT_OK = qw/ &is_peg /;


#########################################################################
=pod

=head1 CONSTRUCTOR

=over 4

=item B<new( %args )>

Creates a new Peg  Returns the new object if there were no problems
creating it, or undef on failure.

=cut

sub new {
    my $class = shift;
    $class    = ref($class) || $class;

    my $self = [\%FIELDS];

    # set instance variables:
    $self->{discs} = [];

    bless $self, $class;
    return $self->_init(@_);
}


#########################################################################
=pod

=item B<_init( %args )>

Initializes instance variables with arguments supplied.  Does a straight
copy (if object attributes are defined).  Returns undef on error, or the
object on success.

Overwrite this if inheriting from this class.

=cut

sub _init {
    my $self = shift;
    my %args = @_;

	foreach (keys(%args)) {
		$self->{$_} = $args{$_};
	}

    return $self;
}



#########################################################################
sub discs {
    my $self = shift;

    return @{$self->{discs}};
}

#########################################################################
=pod

=back

=head1 METHODS

=over 4

=item B<add_disc( Hanoi::Disc )>

Tries to add a disc to the peg.  The disc must be smaller than the top disc
on the peg in order for the operation to succeed.  Returns 1 on success, 0 on
failure & undef on error.

=cut

sub add_disc {
	my $self = shift || return undef;
	my $disc = shift;

	# error checking:
	unless (is_disc($disc)) {
		warn "disc ($disc) is not a disc!";
		return undef;
	}

	if ($self->is_empty()) {
		push (@{$self->{discs}}, $disc);
		return 1;
	}

	# only ok to add if top disc is bigger than new disc...

	my $top_disc = ${$self->{discs}}[-1];
	if ($top_disc->bigger_than( $disc )) {
		push (@{$self->{discs}}, $disc);
		return 1;
	}

	warn "illegal move: disc to add is bigger than top disc on peg!\n";
	return 0;
}


#########################################################################
=pod

=item B<remove_disc()>

Tries to remove the top disc from the peg.  There must be a disc on the peg
to do so - returns this disc on success, 0 on failure, or undef on error.

=cut

sub remove_disc {
	my $self = shift || return undef;

	if ($self->is_empty) {
		warn "illegal move: no disc on peg!\n";
		return 0;
	}

	# only ok to add if top disc is bigger than new disc...
	my $top_disc = pop @{$self->{discs}};

	return $top_disc;
}


#########################################################################
=pod

=item B<is_empty()>

Returns 1 if there are no discs on the peg, 0 if there are, and undef on
error.

=cut

sub is_empty {
	my $self = shift || return undef;

	return 0 if (scalar @{$self->{discs}});

	return 1;
}


#########################################################################
=pod

=back

=head1 SUBROUTINES

=over 4

=item B<is_peg( $variable )>

Returns true value if $variable is a member of the I<Hanoi::Peg> class (or sub-class),
and a false falue otherwise.

Exported with @EXPORT_OK.

=cut

sub is_peg {
    my $variable = shift || return;	# should you print a warning here?
    return UNIVERSAL::isa($variable, 'Hanoi::Peg');
}


#########################################################################
# Embedded test script...
package main;
unless (caller) {
	my Hanoi::Peg $p1 = new Hanoi::Peg() || die "couldn't create peg!";
	my Hanoi::Disc ($d1, $d2, $d3);

	print "disc sizes:\n";
	$d1 = new Hanoi::Disc( Size => int(rand()*10) )
	  || die "couldn't create disc!";
	print "\tD1 = ", $d1->get_size(), "\n";
	$d2 = new Hanoi::Disc( Size => int(rand()*10) )
	  || die "couldn't create disc!";
	print "\tD2 = ", $d2->get_size(), "\n";
	$d3 = new Hanoi::Disc( Size => int(rand()*10) )
	  || die "couldn't create disc!";
	print "\tD3 = ", $d3->get_size(), "\n";

	
	print "trying to remove nothing (should get warning)...\n";
	$p1->remove_disc();

	print "addink disc 1:";
	$p1->add_disc($d1) || die "couldn't add disc 1!";
	print " ok\naddink disc 2:";
	if ( $p1->add_disc($d2) ) {
		print " ok\nremovink disc 2:";
		my $rdisc = $p1->remove_disc();
		unless ( $rdisc == $d2 ) {
			die "disc removed (", $rdisc->get_size,
			    ") not the same as disc just added (",
			    $d2->get_size, ")...";
		}
	}
	print " ok\naddink disc 3:";
	$p1->add_disc($d3) || die "couldn't add disc 3!";
	print " ok\n";

	# access illegal variable (will die...):
	# $p1->{error} = 1;
}



1;

__END__

#########################################################################
=pod

=back

=head1 INHERITS FROM

 Exporter

=head1 DEPENDANCIES

 Exporter
 Hanoi::Disc

=head1 EXPORTS

On demand:
 is_peg()

=head1 NOTES

 none.

=head1 TODO

 nothing.

=head1 AUTHORS

 Steve Purkis <spurkis@redhotchilli.com>

=head1 SEE ALSO

Hanoi::Peg;

=cut
